@@ -39,6 +39,7 @@
 #include "local.h"
 #include "memory.h"
 #include "nameserv_async.h"
+#include "privops.h"
 #include "sched.h"
 
 /* ================================================== */
@@ -117,8 +118,8 @@ static void rehash_records(void);
 static void clean_source_record(SourceRecord *record);
 
 static void
-slew_sources(struct timeval *raw,
-             struct timeval *cooked,
+slew_sources(struct timespec *raw,
+             struct timespec *cooked,
              double dfreq,
              double doffset,
              LCL_ChangeType change_type,
@@ -354,7 +355,7 @@ replace_source(NTP_Remote_Address *old_addr, NTP_Remote_Address *new_addr)
   /* The hash table must be rebuilt for the new address */
   rehash_records();
 
-  LOG(LOGS_INFO, LOGF_NtpSources, "Source %s replaced with %s",
+  LOG(LOGS_INFO, "Source %s replaced with %s",
       UTI_IPToString(&old_addr->ip_addr),
       UTI_IPToString(&new_addr->ip_addr));
 
@@ -377,7 +378,7 @@ process_resolved_name(struct UnresolvedSource *us, IPAddr *ip_addrs, int n_addrs
     address.ip_addr = ip_addrs[((unsigned int)i + first) % n_addrs];
     address.port = us->port;
 
-    DEBUG_LOG(LOGF_NtpSources, "(%d) %s", i + 1, UTI_IPToString(&address.ip_addr));
+    DEBUG_LOG("(%d) %s", i + 1, UTI_IPToString(&address.ip_addr));
 
     if (us->replacement) {
       if (replace_source(&us->replace_source, &address) != NSR_AlreadyInUse)
@@ -404,7 +405,7 @@ name_resolve_handler(DNS_Status status, int n_addrs, IPAddr *ip_addrs, void *any
 
   assert(us == resolving_source);
 
-  DEBUG_LOG(LOGF_NtpSources, "%s resolved to %d addrs", us->name, n_addrs);
+  DEBUG_LOG("%s resolved to %d addrs", us->name, n_addrs);
 
   switch (status) {
     case DNS_TryAgain:
@@ -413,7 +414,7 @@ name_resolve_handler(DNS_Status status, int n_addrs, IPAddr *ip_addrs, void *any
       process_resolved_name(us, ip_addrs, n_addrs);
       break;
     case DNS_Failure:
-      LOG(LOGS_WARN, LOGF_NtpSources, "Invalid host %s", us->name);
+      LOG(LOGS_WARN, "Invalid host %s", us->name);
       break;
     default:
       assert(0);
@@ -438,7 +439,7 @@ name_resolve_handler(DNS_Status status, int n_addrs, IPAddr *ip_addrs, void *any
 
   if (next) {
     /* Continue with the next source in the list */
-    DEBUG_LOG(LOGF_NtpSources, "resolving %s", next->name);
+    DEBUG_LOG("resolving %s", next->name);
     DNS_Name2IPAddressAsync(next->name, name_resolve_handler, next);
   } else {
     /* This was the last source in the list. If some sources couldn't
@@ -469,14 +470,14 @@ resolve_sources(void *arg)
 
   assert(!resolving_source);
 
-  DNS_Reload();
+  PRV_ReloadDNS();
 
   /* Start with the first source in the list, name_resolve_handler
      will iterate over the rest */
   us = unresolved_sources;
 
   resolving_source = us;
-  DEBUG_LOG(LOGF_NtpSources, "resolving %s", us->name);
+  DEBUG_LOG("resolving %s", us->name);
   DNS_Name2IPAddressAsync(us->name, name_resolve_handler, us);
 }
 
@@ -657,8 +658,7 @@ resolve_source_replacement(SourceRecord *record)
 {
   struct UnresolvedSource *us;
 
-  DEBUG_LOG(LOGF_NtpSources, "trying to replace %s",
-            UTI_IPToString(&record->remote_addr->ip_addr));
+  DEBUG_LOG("trying to replace %s", UTI_IPToString(&record->remote_addr->ip_addr));
 
   us = MallocNew(struct UnresolvedSource);
   us->name = Strdup(record->name);
@@ -680,8 +680,8 @@ resolve_source_replacement(SourceRecord *record)
 void
 NSR_HandleBadSource(IPAddr *address)
 {
-  static struct timeval last_replacement;
-  struct timeval now;
+  static struct timespec last_replacement;
+  struct timespec now;
   NTP_Remote_Address remote_addr;
   SourceRecord *record;
   int slot, found;
@@ -702,9 +702,9 @@ NSR_HandleBadSource(IPAddr *address)
 
   /* Don't resolve names too frequently */
   SCH_GetLastEventTime(NULL, NULL, &now);
-  UTI_DiffTimevalsToDouble(&diff, &now, &last_replacement);
+  diff = UTI_DiffTimespecsToDouble(&now, &last_replacement);
   if (fabs(diff) < RESOLVE_INTERVAL_UNIT * (1 << MIN_REPLACEMENT_INTERVAL)) {
-    DEBUG_LOG(LOGF_NtpSources, "replacement postponed");
+    DEBUG_LOG("replacement postponed");
     return;
   }
   last_replacement = now;
@@ -742,7 +742,7 @@ static void remove_tentative_pool_sources(int pool)
     if (!record->remote_addr || record->pool != pool || !record->tentative)
       continue;
 
-    DEBUG_LOG(LOGF_NtpSources, "removing tentative source %s",
+    DEBUG_LOG("removing tentative source %s",
               UTI_IPToString(&record->remote_addr->ip_addr));
 
     clean_source_record(record);
@@ -776,7 +776,8 @@ NSR_GetLocalRefid(IPAddr *address)
 /* This routine is called by ntp_io when a new packet arrives off the network,
    possibly with an authentication tail */
 void
-NSR_ProcessReceive(NTP_Packet *message, struct timeval *now, double now_err, NTP_Remote_Address *remote_addr, NTP_Local_Address *local_addr, int length)
+NSR_ProcessRx(NTP_Remote_Address *remote_addr, NTP_Local_Address *local_addr,
+              NTP_Local_Timestamp *rx_ts, NTP_Packet *message, int length)
 {
   SourceRecord *record;
   struct SourcePool *pool;
@@ -788,19 +789,18 @@ NSR_ProcessReceive(NTP_Packet *message, struct timeval *now, double now_err, NTP
   if (found == 2) { /* Must match IP address AND port number */
     record = get_record(slot);
 
-    if (!NCR_ProcessKnown(message, now, now_err, record->data, local_addr, length))
+    if (!NCR_ProcessRxKnown(record->data, local_addr, rx_ts, message, length))
       return;
 
     if (record->tentative) {
-      /* This was the first valid reply from the source */
+      /* This was the first good reply from the source */
       record->tentative = 0;
 
       if (record->pool != INVALID_POOL) {
         pool = ARR_GetElement(pools, record->pool);
         pool->sources++;
 
-        DEBUG_LOG(LOGF_NtpSources, "pool %s has %d confirmed sources",
-                  record->name, pool->sources);
+        DEBUG_LOG("pool %s has %d confirmed sources", record->name, pool->sources);
 
         /* If the number of sources from the pool reached the configured
            maximum, remove the remaining tentative sources */
@@ -809,15 +809,34 @@ NSR_ProcessReceive(NTP_Packet *message, struct timeval *now, double now_err, NTP
       }
     }
   } else {
-    NCR_ProcessUnknown(message, now, now_err, remote_addr, local_addr, length);
+    NCR_ProcessRxUnknown(remote_addr, local_addr, rx_ts, message, length);
+  }
+}
+
+/* ================================================== */
+
+void
+NSR_ProcessTx(NTP_Remote_Address *remote_addr, NTP_Local_Address *local_addr,
+              NTP_Local_Timestamp *tx_ts, NTP_Packet *message, int length)
+{
+  SourceRecord *record;
+  int slot, found;
+
+  find_slot(remote_addr, &slot, &found);
+
+  if (found == 2) { /* Must match IP address AND port number */
+    record = get_record(slot);
+    NCR_ProcessTxKnown(record->data, local_addr, tx_ts, message, length);
+  } else {
+    NCR_ProcessTxUnknown(remote_addr, local_addr, tx_ts, message, length);
   }
 }
 
 /* ================================================== */
 
 static void
-slew_sources(struct timeval *raw,
-             struct timeval *cooked,
+slew_sources(struct timespec *raw,
+             struct timespec *cooked,
              double dfreq,
              double doffset,
              LCL_ChangeType change_type,
@@ -831,6 +850,7 @@ slew_sources(struct timeval *raw,
     if (record->remote_addr) {
       if (change_type == LCL_ChangeUnknownStep) {
         NCR_ResetInstance(record->data);
+        NCR_ResetPoll(record->data);
       } else {
         NCR_SlewTimes(record->data, cooked, dfreq, doffset);
       }
@@ -1083,7 +1103,7 @@ NSR_InitiateSampleBurst(int n_good_samples, int n_total_samples,
    identify the source record. */
 
 void
-NSR_ReportSource(RPT_SourceReport *report, struct timeval *now)
+NSR_ReportSource(RPT_SourceReport *report, struct timespec *now)
 {
   NTP_Remote_Address rem_addr;
   int slot, found;
@@ -1099,6 +1119,26 @@ NSR_ReportSource(RPT_SourceReport *report, struct timeval *now)
   }
 }
 
+/* ================================================== */
+/* The ip address is assumed to be completed on input, that is how we
+   identify the source record. */
+
+int
+NSR_GetNTPReport(RPT_NTPReport *report)
+{
+  NTP_Remote_Address rem_addr;
+  int slot, found;
+
+  rem_addr.ip_addr = report->remote_addr;
+  rem_addr.port = 0;
+  find_slot(&rem_addr, &slot, &found);
+  if (!found)
+    return 0;
+
+  NCR_GetNTPReport(get_record(slot)->data, report);
+  return 1;
+}
+
 /* ================================================== */
 
 void
