@@ -42,25 +42,30 @@ typedef struct {
   int max_sources;
   int min_samples;
   int max_samples;
+  int interleaved;
   int sel_options;
   uint32_t authkey;
   double max_delay;
   double max_delay_ratio;
   double max_delay_dev_ratio;
+  double min_delay;
+  double asymmetry;
+  double offset;
 } SourceParameters;
 
 #define SRC_DEFAULT_PORT 123
 #define SRC_DEFAULT_MINPOLL 6
 #define SRC_DEFAULT_MAXPOLL 10
-#define SRC_DEFAULT_PRESEND_MINPOLL 0
+#define SRC_DEFAULT_PRESEND_MINPOLL 100
 #define SRC_DEFAULT_MAXDELAY 3.0
 #define SRC_DEFAULT_MAXDELAYRATIO 0.0
 #define SRC_DEFAULT_MAXDELAYDEVRATIO 10.0
 #define SRC_DEFAULT_MINSTRATUM 0
-#define SRC_DEFAULT_POLLTARGET 6
+#define SRC_DEFAULT_POLLTARGET 8
 #define SRC_DEFAULT_MAXSOURCES 4
 #define SRC_DEFAULT_MINSAMPLES (-1)
 #define SRC_DEFAULT_MAXSAMPLES (-1)
+#define SRC_DEFAULT_ASYMMETRY 1.0
 #define INACTIVE_AUTHKEY 0
 
 /* Flags for source selection */
