@@ -46,4 +46,9 @@ extern int SYS_Linux_OpenPHC(const char *path, int phc_index);
 extern int SYS_Linux_GetPHCSample(int fd, int nocrossts, double precision, int *reading_mode,
                                   struct timespec *phc_ts, struct timespec *sys_ts, double *err);
 
+extern int SYS_Linux_SetPHCExtTimestamping(int fd, int pin, int channel,
+                                           int rising, int falling, int enable);
+
+extern int SYS_Linux_ReadPHCExtTimestamp(int fd, struct timespec *phc_ts, int *channel);
+
 #endif  /* GOT_SYS_LINUX_H */
